@@ -1,5 +1,5 @@
 # file MASS/confint.q
-# copyright (C) 1994-2000 W. N. Venables and B. D. Ripley
+# copyright (C) 1994-2005 W. N. Venables and B. D. Ripley
 #
 
 confint.glm <- function(object, parm, level = 0.95, trace = FALSE, ...)
@@ -8,6 +8,7 @@ confint.glm <- function(object, parm, level = 0.95, trace = FALSE, ...)
     if(missing(parm)) parm <- seq(along=pnames)
     else if(is.character(parm))  parm <- match(parm, pnames, nomatch = 0)
     cat("Waiting for profiling to be done...\n")
+    flush.console()
     object <- profile(object, which = parm, alpha = (1. - level)/4.,
                       trace = trace)
     confint(object, parm=parm, level=level, trace=trace, ...)
@@ -36,11 +37,13 @@ confint.profile.glm <-
 }
 
 confint.nls <-
-  function(object, parm = seq(along=pnames), level = 0.95, ...)
+  function(object, parm, level = 0.95, ...)
 {
   pnames <- names(coef(object))
+  if(missing(parm)) parm <- seq(along=pnames)
   if(is.character(parm))  parm <- match(parm, pnames, nomatch = 0)
   cat("Waiting for profiling to be done...\n")
+  flush.console()
   object <- profile(object, which = parm, alphamax = (1. - level)/4.)
   confint(object, parm=parm, level=level, ...)
 }
