@@ -68,7 +68,7 @@ qda.default <-
            method = c("moment", "mle", "mve", "t"),
            CV = FALSE, nu = 5, ...)
 {
-    if(is.null(dim(x))) stop("x is not a matrix")
+    if(is.null(dim(x))) stop("'x' is not a matrix")
     x <- as.matrix(x)
     n <- nrow(x)
     p <- ncol(x)
@@ -82,7 +82,7 @@ qda.default <-
     ng <- length(proportions)
 # allow for supplied prior
     if(any(prior < 0) || round(sum(prior), 5) != 1) stop("invalid prior")
-    if(length(prior) != ng) stop("prior is of incorrect length")
+    if(length(prior) != ng) stop("'prior' is of incorrect length")
     names(prior) <- lev
 # means by group (rows) and variable (columns)
     group.means <- tapply(x, list(rep(g, ncol(x)), col(x)), mean)
@@ -90,7 +90,7 @@ qda.default <-
     ldet <- numeric(ng)
     method <- match.arg(method)
     if(CV && !(method == "moment" || method == "mle"))
-        stop(paste("Cannot use leave-one-out CV with method", method))
+        stop("cannot use leave-one-out CV with method ", sQuote(method))
     for (i in 1:ng){
         if(method == "mve") {
             cX <- cov.mve(x[unclass(g) == i, ])
@@ -99,7 +99,7 @@ qda.default <-
             scaling[, , i] <- sX$v %*% diag(sqrt(1/sX$d),,p)
             ldet[i] <- sum(log(sX$d))
         } else if(method == "t") {
-            if(nu <= 2) stop("nu must exceed 2")
+            if(nu <= 2) stop("'nu' must exceed 2")
             m <- counts[i]
             X <- x[unclass(g) == i, ]
             w <- rep(1, m)
@@ -114,7 +114,7 @@ qda.default <-
                 if(all(abs(w - w0) < 1e-2)) break
             }
             qx <- qr(sqrt(w)*scale(X, center=group.means[i, ], scale=FALSE))
-            if(qx$rank < p) stop(paste("Rank deficiency in group", lev[i]))
+            if(qx$rank < p) stop("rank deficiency in group ", lev[i])
             qx <- qx$qr* sqrt((1 + p/nu)/m)
             scaling[, , i] <- backsolve(qx[1:p,  ], diag(p))
             ldet[i] <- 2*sum(log(abs(diag(qx))))
@@ -122,7 +122,7 @@ qda.default <-
             if(method == "moment") nk <- counts[i] - 1 else nk <- counts[i]
             X <- scale(x[unclass(g) == i, ], center=group.means[i, ], scale=FALSE)/sqrt(nk)
             qx <- qr(X)
-            if(qx$rank < p) stop(paste("Rank deficiency in group", lev[i]))
+            if(qx$rank < p) stop("rank deficiency in group ", lev[i])
             qx <- qx$qr
             scaling[, , i] <- backsolve(qx[1:p, ], diag(p))
             ldet[i] <- 2*sum(log(abs(diag(qx))))
@@ -169,13 +169,18 @@ predict.qda <- function(object, newdata, prior = object$prior,
 			method = c("plug-in", "predictive", "debiased",
                           "looCV"), ...)
 {
-    if(!inherits(object, "qda")) stop("object not of class qda")
+    if(!inherits(object, "qda")) stop("object not of class \"qda\"")
     method <- match.arg(method)
     if(method == "looCV" && !missing(newdata))
-        stop("Cannot have leave-one-out CV with newdata")
+        stop("cannot have leave-one-out CV with 'newdata'")
     if(is.null(mt <- object$call$method)) mt <- "moment"
     if(method == "looCV" && !(mt == "moment" || mt == "mle"))
-        stop(paste("Cannot use leave-one-out CV with method", mt))
+        stop("cannot use leave-one-out CV with method ", sQuote(mt))
+    ngroup <- length(object$prior)
+    if(!missing(prior)) {
+        if(any(prior < 0) || round(sum(prior), 5) != 1) stop("invalid prior")
+        if(length(prior) != ngroup) stop("'prior' is of incorrect length")
+    }
     if(!is.null(Terms <- object$terms)) {
     # formula fit
         if(missing(newdata)) newdata <- model.frame(object)
@@ -220,14 +225,13 @@ predict.qda <- function(object, newdata, prior = object$prior,
     if(ncol(x) != p) stop("wrong number of variables")
     if(length(colnames(x)) > 0 &&
        any(colnames(x) != dimnames(object$means)[[2]]))
-        warning("Variable names in newdata do not match those in object")
-    ngroup <- length(object$prior)
+        warning("variable names in 'newdata' do not match those in 'object'")
     dist <- matrix(0, nrow = nrow(x), ncol = ngroup)
     if(method == "plug-in") {
         for(i in 1:ngroup) {
             dev <- ((x - matrix(object$means[i,  ], nrow(x),
                                 ncol(x), byrow = TRUE)) %*% object$scaling[,,i])
-            dist[, i] <- 0.5 * rowSums(dev^2) + 0.5 * object$ldet[i] - log(object$prior[i])
+            dist[, i] <- 0.5 * rowSums(dev^2) + 0.5 * object$ldet[i] - log(prior[i])
         }
 #        dist <- exp( -(dist - min(dist, na.rm=T)))
         dist <- exp( -(dist - apply(dist, 1, min, na.rm=TRUE)))
@@ -249,7 +253,7 @@ predict.qda <- function(object, newdata, prior = object$prior,
         ldet[ind] <- log(fac) + p * log((nc-NG)/(nc-1-NG)) + ldet[ind]
         dist[ind] <- dist[ind] * (nc^2/(nc-1)^2) * (nc-1-NG)/(nc-NG) / fac
         dist <- 0.5 * dist + 0.5 * ldet -
-            matrix(log(object$prior), n, ngroup, byrow=TRUE)
+            matrix(log(prior), n, ngroup, byrow=TRUE)
         dist <- exp( -(dist - apply(dist, 1, min, na.rm=TRUE)))
     } else if(method == "debiased") {
         for(i in 1:ngroup) {
@@ -258,7 +262,7 @@ predict.qda <- function(object, newdata, prior = object$prior,
             dev <- ((x - matrix(object$means[i,  ], nrow = nrow(x),
                                 ncol = ncol(x), byrow = TRUE)) %*% object$scaling[,,i])
             dist[, i] <- 0.5 * (1 - (p-1)/(nk-1)) * rowSums(dev^2) +
-                0.5 * object$ldet[i] - log(object$prior[i]) + 0.5 * Bm - p/(2*nk)
+                0.5 * object$ldet[i] - log(prior[i]) + 0.5 * Bm - p/(2*nk)
         }
         dist <- exp( -(dist - apply(dist, 1, min, na.rm=TRUE)))
     } else {
@@ -268,11 +272,11 @@ predict.qda <- function(object, newdata, prior = object$prior,
                                 ncol = ncol(x), byrow = TRUE))
                     %*% object$scaling[,,i])
             dev <- 1 + rowSums(dev^2)/(nk+1)
-            dist[, i] <- object$prior[i] * exp(-object$ldet[i]/2) *
+            dist[, i] <- prior[i] * exp(-object$ldet[i]/2) *
                 dev^(-nk/2) * (1 + nk)^(-p/2)
         }
     }
-    posterior <- dist/drop(dist %*% rep(1, length(object$prior)))
+    posterior <- dist/drop(dist %*% rep(1, ngroup))
     cl <- factor(max.col(posterior), levels=seq(along=object$lev),
                  labels=object$lev)
     dimnames(posterior) <- list(rownames(x), object$lev)
